@@ -9,7 +9,10 @@
 #include <sys/mman.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <fcntl.h>
+#include <sys/types.h>
+#include <utime.h>
 #include <alloca.h>
 
 namespace {
@@ -236,6 +239,8 @@ void FileDescriptor::throw_runtime_error(const char* desc)
     throw std::runtime_error(desc);
 }
 
+bool FileDescriptor::is_open() const { return fd != -1; }
+
 void FileDescriptor::close()
 {
     if (fd == -1) return;
@@ -339,6 +344,84 @@ MMap FileDescriptor::mmap(size_t length, int prot, int flags, off_t offset)
     return MMap(res, length);
 }
 
+bool FileDescriptor::ofd_setlk(struct flock& lk)
+{
+#ifdef F_OFD_SETLK
+    if (fcntl(fd, F_OFD_SETLK, &lk) != -1)
+#else
+    if (fcntl(fd, F_SETLK, &lk) != -1)
+#endif
+        return true;
+    if (errno != EAGAIN && errno != EACCES)
+        throw_error("cannot acquire lock");
+    return false;
+}
+
+bool FileDescriptor::ofd_setlkw(struct flock& lk, bool retry_on_signal)
+{
+    while (true)
+    {
+#ifdef F_OFD_SETLK
+        if (fcntl(fd, F_OFD_SETLKW, &lk) != -1)
+#else
+        if (fcntl(fd, F_SETLKW, &lk) != -1)
+#endif
+            return true;
+        if (errno != EINTR)
+            throw_error("cannot acquire lock");
+        if (!retry_on_signal)
+            return false;
+    }
+}
+
+bool FileDescriptor::ofd_getlk(struct flock& lk)
+{
+#ifdef F_OFD_SETLK
+    if (fcntl(fd, F_OFD_GETLK, &lk) == -1)
+#else
+    if (fcntl(fd, F_GETLK, &lk) == -1)
+#endif
+        throw_error("cannot test lock");
+    return lk.l_type == F_UNLCK;
+}
+
+void FileDescriptor::futimens(const struct ::timespec ts[2])
+{
+    if (::futimens(fd, ts) == -1)
+        throw_error("cannot change file timestamps");
+}
+
+void FileDescriptor::fsync()
+{
+    if (::fsync(fd) == -1)
+        throw_error("fsync failed");
+}
+
+void FileDescriptor::fdatasync()
+{
+    if (::fdatasync(fd) == -1)
+        throw_error("fdatasync failed");
+}
+
+
+/*
+ * PreserveFileTimes
+ */
+
+PreserveFileTimes::PreserveFileTimes(FileDescriptor fd)
+    : fd(fd)
+{
+    struct stat st;
+    fd.fstat(st);
+    ts[0] = st.st_atim;
+    ts[1] = st.st_mtim;
+}
+
+PreserveFileTimes::~PreserveFileTimes()
+{
+    fd.futimens(ts);
+}
+
 
 /*
  * NamedFileDescriptor
@@ -374,12 +457,32 @@ void NamedFileDescriptor::throw_runtime_error(const char* desc)
 }
 
 
+/*
+ * ManagedNamedFileDescriptor
+ */
+
+ManagedNamedFileDescriptor::~ManagedNamedFileDescriptor()
+{
+    if (fd != -1) ::close(fd);
+}
+
+ManagedNamedFileDescriptor& ManagedNamedFileDescriptor::operator=(ManagedNamedFileDescriptor&& o)
+{
+    if (&o == this) return *this;
+    close();
+    fd = o.fd;
+    pathname = std::move(o.pathname);
+    o.fd = -1;
+    return *this;
+}
+
+
 /*
  * Path
  */
 
 Path::Path(const char* pathname, int flags)
-    : NamedFileDescriptor(-1, pathname)
+    : ManagedNamedFileDescriptor(-1, pathname)
 {
     fd = open(pathname, flags | O_PATH);
     if (fd == -1)
@@ -387,7 +490,7 @@ Path::Path(const char* pathname, int flags)
 }
 
 Path::Path(const std::string& pathname, int flags)
-    : NamedFileDescriptor(-1, pathname)
+    : ManagedNamedFileDescriptor(-1, pathname)
 {
     fd = open(pathname.c_str(), flags | O_PATH);
     if (fd == -1)
@@ -395,17 +498,11 @@ Path::Path(const std::string& pathname, int flags)
 }
 
 Path::Path(Path& parent, const char* pathname, int flags)
-    : NamedFileDescriptor(parent.openat(pathname, flags | O_PATH),
+    : ManagedNamedFileDescriptor(parent.openat(pathname, flags | O_PATH),
             str::joinpath(parent.name(), pathname))
 {
 }
 
-Path::~Path()
-{
-    if (fd != -1)
-        ::close(fd);
-}
-
 DIR* Path::fdopendir()
 {
     int fd1 = ::openat(fd, ".", O_DIRECTORY);
@@ -440,18 +537,57 @@ int Path::openat(const char* pathname, int flags, mode_t mode)
     return res;
 }
 
+int Path::openat_ifexists(const char* pathname, int flags, mode_t mode)
+{
+    int res = ::openat(fd, pathname, flags, mode);
+    if (res == -1)
+    {
+        if (errno == ENOENT)
+            return -1;
+        throw_error("cannot openat");
+    }
+    return res;
+}
+
+bool Path::faccessat(const char* pathname, int mode, int flags)
+{
+    return ::faccessat(fd, pathname, mode, flags) == 0;
+}
+
 void Path::fstatat(const char* pathname, struct stat& st)
 {
     if (::fstatat(fd, pathname, &st, 0) == -1)
         throw_error("cannot fstatat");
 }
 
+bool Path::fstatat_ifexists(const char* pathname, struct stat& st)
+{
+    if (::fstatat(fd, pathname, &st, 0) == -1)
+    {
+        if (errno == ENOENT)
+            return false;
+        throw_error("cannot fstatat");
+    }
+    return true;
+}
+
 void Path::lstatat(const char* pathname, struct stat& st)
 {
     if (::fstatat(fd, pathname, &st, AT_SYMLINK_NOFOLLOW) == -1)
         throw_error("cannot fstatat");
 }
 
+bool Path::lstatat_ifexists(const char* pathname, struct stat& st)
+{
+    if (::fstatat(fd, pathname, &st, AT_SYMLINK_NOFOLLOW) == -1)
+    {
+        if (errno == ENOENT)
+            return false;
+        throw_error("cannot fstatat");
+    }
+    return true;
+}
+
 void Path::unlinkat(const char* pathname)
 {
     if (::unlinkat(fd, pathname, 0) == -1)
@@ -472,21 +608,11 @@ Path::iterator::iterator(Path& dir)
     : path(&dir)
 {
     this->dir = dir.fdopendir();
-
-    long name_max = fpathconf(dir.fd, _PC_NAME_MAX);
-    if (name_max == -1) // Limit not defined, or error: take a guess
-        name_max = 255;
-    size_t len = offsetof(dirent, d_name) + name_max + 1;
-    cur_entry = (struct dirent*)malloc(len);
-    if (cur_entry == NULL)
-        throw std::bad_alloc();
-
     operator++();
 }
 
 Path::iterator::~iterator()
 {
-    if (cur_entry) free(cur_entry);
     if (dir) closedir(dir);
 }
 
@@ -505,12 +631,12 @@ bool Path::iterator::operator!=(const iterator& i) const
 
 void Path::iterator::operator++()
 {
-    struct dirent* result;
-    if (readdir_r(dir, cur_entry, &result) != 0)
-        path->throw_error("cannot readdir_r");
-
-    if (result == nullptr)
+    errno = 0;
+    cur_entry = readdir(dir);
+    if (cur_entry == nullptr)
     {
+        if (errno) path->throw_error("cannot readdir");
+
         // Turn into an end iterator
         free(cur_entry);
         cur_entry = nullptr;
@@ -614,6 +740,11 @@ bool Path::iterator::issock() const
     return S_ISSOCK(st.st_mode);
 }
 
+Path Path::iterator::open_path(int flags) const
+{
+    return Path(*path, cur_entry->d_name);
+}
+
 
 void Path::rmtree()
 {
@@ -637,23 +768,19 @@ void Path::rmtree()
  */
 
 File::File(const std::string& pathname)
-    : NamedFileDescriptor(-1, pathname)
+    : ManagedNamedFileDescriptor(-1, pathname)
 {
 }
 
 File::File(const std::string& pathname, int flags, mode_t mode)
-    : NamedFileDescriptor(-1, pathname)
+    : ManagedNamedFileDescriptor(-1, pathname)
 {
     open(flags, mode);
 }
 
-File::~File()
-{
-    if (fd != -1) ::close(fd);
-}
-
 void File::open(int flags, mode_t mode)
 {
+    close();
     fd = ::open(pathname.c_str(), flags, mode);
     if (fd == -1)
         throw std::system_error(errno, std::system_category(), "cannot open file " + pathname);
@@ -661,6 +788,7 @@ void File::open(int flags, mode_t mode)
 
 bool File::open_ifexists(int flags, mode_t mode)
 {
+    close();
     fd = ::open(pathname.c_str(), flags, mode);
     if (fd != -1) return true;
     if (errno == ENOENT) return false;
@@ -672,10 +800,24 @@ File File::mkstemp(const std::string& prefix)
     char* fbuf = (char*)alloca(prefix.size() + 7);
     memcpy(fbuf, prefix.data(), prefix.size());
     memcpy(fbuf + prefix.size(), "XXXXXX", 7);
-    int fd = ::mkstemp(fbuf);
+    return mkstemp(fbuf);
+}
+
+File File::mkstemp(const char* prefix)
+{
+    size_t prefix_size = strlen(prefix);
+    char* fbuf = (char*)alloca(prefix_size + 7);
+    memcpy(fbuf, prefix, prefix_size);
+    memcpy(fbuf + prefix_size, "XXXXXX", 7);
+    return mkstemp(fbuf);
+}
+
+File File::mkstemp(char* pathname_template)
+{
+    int fd = ::mkstemp(pathname_template);
     if (fd < 0)
-        throw std::system_error(errno, std::system_category(), std::string("cannot create temporary file ") + fbuf);
-    return File(fd, fbuf);
+        throw std::system_error(errno, std::system_category(), std::string("cannot create temporary file ") + pathname_template);
+    return File(fd, pathname_template);
 }
 
 std::string read_file(const std::string& file)
@@ -686,6 +828,9 @@ std::string read_file(const std::string& file)
     struct stat st;
     in.fstat(st);
 
+    if (st.st_size == 0)
+        return std::string();
+
     // mmap the input file
     MMap src = in.mmap(st.st_size, PROT_READ, MAP_SHARED);
 
@@ -699,7 +844,7 @@ void write_file(const std::string& file, const std::string& data, mode_t mode)
 
 void write_file(const std::string& file, const void* data, size_t size, mode_t mode)
 {
-    File out(file, O_WRONLY | O_CREAT, mode);
+    File out(file, O_WRONLY | O_CREAT | O_TRUNC, mode);
     out.write_all_or_retry(data, size);
     out.close();
 }
@@ -723,7 +868,7 @@ void write_file_atomically(const std::string& file, const void* data, size_t siz
     out.write_all_or_retry(data, size);
     out.close();
 
-    if (rename(out.name().c_str(), file.c_str()) < 0)
+    if (::rename(out.name().c_str(), file.c_str()) < 0)
         throw std::system_error(errno, std::system_category(), "cannot rename " + out.name() + " to " + file);
 }
 
@@ -749,6 +894,12 @@ bool unlink_ifexists(const std::string& file)
         return true;
 }
 
+void rename(const std::string& src_pathname, const std::string& dst_pathname)
+{
+    if (::rename(src_pathname.c_str(), dst_pathname.c_str()) != 0)
+        throw std::system_error(errno, std::system_category(), "cannot rename " + src_pathname + " to " + dst_pathname);
+}
+
 bool rename_ifexists(const std::string& src, const std::string& dst)
 {
     if (::rename(src.c_str(), dst.c_str()) != 0)
@@ -762,6 +913,14 @@ bool rename_ifexists(const std::string& src, const std::string& dst)
         return true;
 }
 
+void touch(const std::string& pathname, time_t ts)
+{
+    struct utimbuf t = { ts, ts };
+    if (::utime(pathname.c_str(), &t) != 0)
+        throw std::system_error(errno, std::system_category(), "cannot set mtime/atime of " + pathname);
+}
+
+
 template<typename String>
 static bool impl_mkdir_ifmissing(String pathname, mode_t mode)
 {
@@ -872,13 +1031,97 @@ void rmtree(const std::string& pathname)
     path.rmtree();
 }
 
-#if 0
-std::string mkdtemp( std::string tmpl )
+bool rmtree_ifexists(const std::string& pathname)
 {
-    char *_tmpl = reinterpret_cast< char * >( alloca( tmpl.size() + 1 ) );
-    strcpy( _tmpl, tmpl.c_str() );
-    return ::mkdtemp( _tmpl );
+    int fd = open(pathname.c_str(), O_PATH);
+    if (fd == -1)
+    {
+        if (errno == ENOENT)
+            return false;
+        throw std::system_error(errno, std::system_category(), "cannot open path " + pathname);
+    }
+    Path path(fd, pathname);
+    path.rmtree();
+    return true;
 }
-#endif
+
+void clock_gettime(::clockid_t clk_id, struct ::timespec& ts)
+{
+    int res = ::clock_gettime(clk_id, &ts);
+    if (res == -1)
+        throw std::system_error(errno, std::system_category(), "clock_gettime failed on clock " + std::to_string(clk_id));
+}
+
+unsigned long long timesec_elapsed(const struct ::timespec& begin, const struct ::timespec& until)
+{
+    if (begin.tv_sec > until.tv_sec)
+        return 0;
+
+    if (begin.tv_sec == until.tv_sec)
+    {
+        if (begin.tv_nsec > until.tv_nsec)
+            return 0;
+        return until.tv_nsec - begin.tv_nsec;
+    }
+
+    if (until.tv_nsec < begin.tv_nsec)
+        return (until.tv_sec - begin.tv_sec - 1) * 1000000000 + (until.tv_nsec + 1000000000 - begin.tv_nsec);
+    else
+        return (until.tv_sec - begin.tv_sec) * 1000000000 + until.tv_nsec - begin.tv_nsec;
+}
+
+/*
+ * Clock
+ */
+
+Clock::Clock(clockid_t clk_id)
+    : clk_id(clk_id)
+{
+    clock_gettime(clk_id, ts);
+}
+
+unsigned long long Clock::elapsed()
+{
+    struct timespec cur_ts;
+    clock_gettime(clk_id, cur_ts);
+    return timesec_elapsed(ts, cur_ts);
+}
+
+
+/*
+ * rlimit
+ */
+
+void getrlimit(int resource, struct ::rlimit& rlim)
+{
+    if (::getrlimit(resource, &rlim) == -1)
+        throw std::system_error(errno, std::system_category(), "getrlimit failed");
+}
+
+void setrlimit(int resource, const struct ::rlimit& rlim)
+{
+    if (::setrlimit(resource, &rlim) == -1)
+        throw std::system_error(errno, std::system_category(), "setrlimit failed");
+}
+
+OverrideRlimit::OverrideRlimit(int resource, rlim_t rlim)
+    : resource(resource)
+{
+    getrlimit(resource, orig);
+    set(rlim);
+}
+
+OverrideRlimit::~OverrideRlimit()
+{
+    setrlimit(resource, orig);
+}
+
+void OverrideRlimit::set(rlim_t rlim)
+{
+    struct rlimit newval(orig);
+    newval.rlim_cur = rlim;
+    setrlimit(resource, newval);
+}
+
 }
 }
